@@ -22,11 +22,11 @@ extern "C" {
 struct Metadata;
 struct WebPPicture;
 
-// Reads a JPEG from 'in_file', returning the decoded output in 'pic'.
-// The output is RGB.
+// Reads a JPEG from 'data', returning the decoded output in 'pic'.
+// The output is RGB or YUV depending on pic->use_argb value.
 // Returns true on success.
-int ReadJPEG(FILE* in_file, struct WebPPicture* const pic,
-             struct Metadata* const metadata);
+int ReadJPEG(const uint8_t* const data, size_t data_size,
+             struct WebPPicture* const pic, struct Metadata* const metadata);
 
 #ifdef __cplusplus
 }    // extern "C"
